@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2017, Kovid Goyal <kovid at kovidgoyal.net>
 
 #
@@ -8,12 +7,10 @@
 #
 
 import math
-from functools import partial as p, wraps
+from functools import lru_cache, wraps
+from functools import partial as p
 from itertools import repeat
-from typing import (
-    Any, Callable, Dict, Generator, Iterable, List, MutableSequence, Optional,
-    Sequence, Tuple, cast
-)
+from typing import Any, Callable, Dict, Iterable, Iterator, List, MutableSequence, Optional, Sequence, Tuple
 
 scale = (0.001, 1., 1.5, 2.)
 _dpi = 96.0
@@ -60,17 +57,17 @@ def half_vline(buf: BufType, width: int, height: int, level: int = 1, which: str
 
 
 def get_holes(sz: int, hole_sz: int, num: int) -> List[Tuple[int, ...]]:
-    if num == 1:
-        pts = [sz // 2]
-    elif num == 2:
-        ssz = (sz - 2 * hole_sz) // 3
-        pts = [ssz + hole_sz // 2, 2 * ssz + hole_sz // 2 + hole_sz]
-    elif num == 3:
-        ssz = (sz - 3 * hole_sz) // 4
-        pts = [ssz + hole_sz // 2, 2 * ssz + hole_sz // 2 + hole_sz, 3 * ssz + 2 * hole_sz + hole_sz // 2]
+    all_holes_use = (num + 1) * hole_sz
+    individual_block_size = (sz - all_holes_use) // (num + 1)
+    half_hole_sz = hole_sz // 2
+    pos = - half_hole_sz
     holes = []
-    for c in pts:
-        holes.append(tuple(range(c - hole_sz // 2, c - hole_sz // 2 + hole_sz)))
+    while pos < sz:
+        left = max(0, pos)
+        right = min(sz, pos + hole_sz)
+        if right > left:
+            holes.append(tuple(range(left, right)))
+        pos = right + individual_block_size
     return holes
 
 
@@ -165,14 +162,15 @@ def downsample(src: BufType, dest: BufType, dest_width: int, dest_height: int, f
             dest[offset + x] = min(255, dest[offset + x] + average_intensity_in_src(x, y))
 
 
-def supersampled(supersample_factor: int = 4) -> Callable:
+class SSByteArray(bytearray):
+    supersample_factor = 1
+
+
+def supersampled(supersample_factor: int = 4) -> Callable[[Callable[..., None]], Callable[..., None]]:
     # Anti-alias the drawing performed by the wrapped function by
     # using supersampling
 
-    class SSByteArray(bytearray):
-        supersample_factor = 1
-
-    def create_wrapper(f: Callable) -> Callable:
+    def create_wrapper(f: Callable[..., None]) -> Callable[..., None]:
         @wraps(f)
         def supersampled_wrapper(buf: BufType, width: int, height: int, *args: Any, **kw: Any) -> None:
             w, h = supersample_factor * width, supersample_factor * height
@@ -203,7 +201,7 @@ def line_equation(x1: int, y1: int, x2: int, y2: int) -> Callable[[int], float]:
 
 
 @supersampled()
-def triangle(buf: BufType, width: int, height: int, left: bool = True) -> None:
+def triangle(buf: SSByteArray, width: int, height: int, left: bool = True) -> None:
     ay1, by1, y2 = 0, height - 1, height // 2
     if left:
         x1, x2 = 0, width - 1
@@ -216,7 +214,7 @@ def triangle(buf: BufType, width: int, height: int, left: bool = True) -> None:
 
 
 @supersampled()
-def corner_triangle(buf: BufType, width: int, height: int, corner: str) -> None:
+def corner_triangle(buf: SSByteArray, width: int, height: int, corner: str) -> None:
     if corner == 'top-right' or corner == 'bottom-left':
         diagonal_y = line_equation(0, 0, width - 1, height - 1)
         if corner == 'top-right':
@@ -233,7 +231,7 @@ def corner_triangle(buf: BufType, width: int, height: int, corner: str) -> None:
 
 
 @supersampled()
-def half_triangle(buf: BufType, width: int, height: int, which: str = 'left', inverted: bool = False) -> None:
+def half_triangle(buf: SSByteArray, width: int, height: int, which: str = 'left', inverted: bool = False) -> None:
     mid_x, mid_y = width // 2, height // 2
     if which == 'left':
         upper_y = line_equation(0, 0, mid_x, mid_y)
@@ -274,19 +272,17 @@ def thick_line(buf: BufType, width: int, height: int, thickness_in_pixels: int,
 
 
 @supersampled()
-def cross_line(buf: BufType, width: int, height: int, left: bool = True, level: int = 1) -> None:
+def cross_line(buf: SSByteArray, width: int, height: int, left: bool = True, level: int = 1) -> None:
     if left:
         p1, p2 = (0, 0), (width - 1, height - 1)
     else:
         p1, p2 = (width - 1, 0), (0, height - 1)
-    supersample_factor = getattr(buf, 'supersample_factor')
-    thick_line(buf, width, height, supersample_factor * thickness(level), p1, p2)
+    thick_line(buf, width, height, buf.supersample_factor * thickness(level), p1, p2)
 
 
 @supersampled()
-def half_cross_line(buf: BufType, width: int, height: int, which: str = 'tl', level: int = 1) -> None:
-    supersample_factor = getattr(buf, 'supersample_factor')
-    thickness_in_pixels = thickness(level) * supersample_factor
+def half_cross_line(buf: SSByteArray, width: int, height: int, which: str = 'tl', level: int = 1) -> None:
+    thickness_in_pixels = thickness(level) * buf.supersample_factor
     my = (height - 1) // 2
     if which == 'tl':
         p1 = 0, 0
@@ -304,9 +300,8 @@ def half_cross_line(buf: BufType, width: int, height: int, which: str = 'tl', le
 
 
 @supersampled()
-def mid_lines(buf: BufType, width: int, height: int, level: int = 1, pts: Iterable[str] = ('lt',)) -> None:
+def mid_lines(buf: SSByteArray, width: int, height: int, level: int = 1, pts: Iterable[str] = ('lt',)) -> None:
     mid_x, mid_y = width // 2, height // 2
-    supersample_factor = getattr(buf, 'supersample_factor')
 
     def pt_to_coords(p: str) -> Tuple[int, int]:
         if p == 'l':
@@ -317,18 +312,19 @@ def mid_lines(buf: BufType, width: int, height: int, level: int = 1, pts: Iterab
             return width - 1, mid_y
         if p == 'b':
             return mid_x, height - 1
+        raise KeyError(f'Unknown p: {p}')
 
     for x in pts:
         p1, p2 = map(pt_to_coords, x)
-        thick_line(buf, width, height, supersample_factor * thickness(level), p1, p2)
+        thick_line(buf, width, height, buf.supersample_factor * thickness(level), p1, p2)
 
 
-BezierFunc = Callable[[float], float]
+ParameterizedFunc = Callable[[float], float]
 
 
-def cubic_bezier(start: Tuple[int, int], end: Tuple[int, int], c1: Tuple[int, int], c2: Tuple[int, int]) -> Tuple[BezierFunc, BezierFunc]:
+def cubic_bezier(start: Tuple[int, int], end: Tuple[int, int], c1: Tuple[int, int], c2: Tuple[int, int]) -> Tuple[ParameterizedFunc, ParameterizedFunc]:
 
-    def bezier_eq(p0: int, p1: int, p2: int, p3: int) -> BezierFunc:
+    def bezier_eq(p0: int, p1: int, p2: int, p3: int) -> ParameterizedFunc:
 
         def f(t: float) -> float:
             tm1 = 1 - t
@@ -356,7 +352,7 @@ def find_bezier_for_D(width: int, height: int) -> int:
         cx += 1
 
 
-def get_bezier_limits(bezier_x: BezierFunc, bezier_y: BezierFunc) -> Generator[Tuple[float, float], None, int]:
+def get_bezier_limits(bezier_x: ParameterizedFunc, bezier_y: ParameterizedFunc) -> Iterator[Tuple[float, float]]:
     start_x = int(bezier_x(0))
     max_x = int(bezier_x(0.5))
     last_t, t_limit = 0., 0.5
@@ -374,7 +370,7 @@ def get_bezier_limits(bezier_x: BezierFunc, bezier_y: BezierFunc) -> Generator[T
             if q > x:
                 increment /= 2
                 if increment < 1e-6:
-                    raise ValueError('Failed to find t for x={}'.format(x))
+                    raise ValueError(f'Failed to find t for x={x}')
             else:
                 start_t += increment
                 increment = t_limit - start_t
@@ -391,7 +387,7 @@ def get_bezier_limits(bezier_x: BezierFunc, bezier_y: BezierFunc) -> Generator[T
 
 
 @supersampled()
-def D(buf: BufType, width: int, height: int, left: bool = True) -> None:
+def D(buf: SSByteArray, width: int, height: int, left: bool = True) -> None:
     c1x = find_bezier_for_D(width, height)
     start = (0, 0)
     end = (0, height - 1)
@@ -411,14 +407,22 @@ def D(buf: BufType, width: int, height: int, left: bool = True) -> None:
                 buf[offset + dest_x] = mbuf[offset + src_x]
 
 
-def draw_parametrized_curve(buf: BufType, width: int, height: int, delta: int, extra: int, xfunc: BezierFunc, yfunc: BezierFunc) -> None:
-    num_samples = height*4
+def draw_parametrized_curve(
+    buf: SSByteArray, width: int, height: int, level: int,
+    xfunc: ParameterizedFunc, yfunc: ParameterizedFunc
+) -> None:
+    supersample_factor = buf.supersample_factor
+    num_samples = height * 8
+    delta, extra = divmod(thickness(level), 2)
+    delta *= supersample_factor
+    extra *= supersample_factor
     seen = set()
     for i in range(num_samples + 1):
-        t = (i / num_samples)
-        p = x_p, y_p = int(xfunc(t)), int(yfunc(t))
+        t = i / num_samples
+        p = int(xfunc(t)), int(yfunc(t))
         if p in seen:
             continue
+        x_p, y_p = p
         seen.add(p)
         for y in range(y_p - delta, y_p + delta + extra):
             if 0 <= y < height:
@@ -429,34 +433,83 @@ def draw_parametrized_curve(buf: BufType, width: int, height: int, delta: int, e
                         buf[pos] = min(255, buf[pos] + 255)
 
 
+def rectircle_equations(
+    cell_width: int, cell_height: int, supersample_factor: int,
+    which: str = '╭'
+) -> Tuple[ParameterizedFunc, ParameterizedFunc]:
+    '''
+    Return two functions, x(t) and y(t) that map the parameter t which must be
+    in the range [0, 1] to x and y co-ordinates in the cell. The rectircle equation
+    we use is:
+
+    (|x| / a) ^ (2a / r) + (|y| / a) ^ (2b / r) = 1
+
+    where 2a = width, 2b = height and r is radius
+
+    The entire rectircle fits in four cells, each cell being one quadrant
+    of the full rectircle and the origin being the center of the rectircle.
+    The functions we return do the mapping for the specified cell.
+    ╭╮
+    ╰╯
+    See https://math.stackexchange.com/questions/1649714
+    '''
+    a = ((cell_width // supersample_factor) // 2) * supersample_factor
+    b = ((cell_height // supersample_factor) // 2) * supersample_factor
+    radius = cell_width / 2
+    yexp = cell_height / radius
+    xexp = radius / cell_width
+    pow = math.pow
+    left_quadrants, lower_quadrants = {'╭': (True, False), '╮': (False, False), '╰': (True, True), '╯': (False, True)}[which]
+    cell_width_is_odd = (cell_width // supersample_factor) % 2
+    adjust_x = cell_width_is_odd * supersample_factor
+
+    if lower_quadrants:
+        def y(t: float) -> float:  # 0 -> top of cell, 1 -> middle of cell
+            return t * b
+    else:
+        def y(t: float) -> float:  # 0 -> bottom of cell, 1 -> middle of cell
+            return (2 - t) * b
+
+    # x(t). To get this we first need |y(t)|/b. This is just t since as t goes
+    # from 0 to 1 y goes from either 0 to b or 0 to -b
+    if left_quadrants:
+        def x(t: float) -> float:
+            xterm = 1 - pow(t, yexp)
+            return math.floor(cell_width - abs(a * pow(xterm, xexp)) - adjust_x)
+    else:
+        def x(t: float) -> float:
+            xterm = 1 - pow(t, yexp)
+            return math.ceil(abs(a * pow(xterm, xexp)))
+
+    return x, y
+
+
 @supersampled()
-def rounded_corner(buf: BufType, width: int, height: int, level: int = 1, which: str = '╭') -> None:
-    supersample_factor = getattr(buf, 'supersample_factor')
-    delta, extra = divmod(thickness(level), 2)
-    hw = ((width / supersample_factor) // 2) * supersample_factor
-    hh = ((height / supersample_factor) // 2) * supersample_factor
-    if which == '╭':
-        start = hw, height - 1
-        end = width - 1, hh
-        c1 = hw, int(0.75 * height)
-        c2 = hw, hh + 1
-    elif which == '╮':
-        start = 0, hh
-        end = hw, height - 1
-        c1 = hw, hh + 1
-        c2 = hw, int(0.75 * height)
-    elif which == '╰':
-        start = width // 2, 0
-        end = width - 1, hh
-        c1 = hw, int(0.25 * height)
-        c2 = hw, hh - 1
-    elif which == '╯':
-        start = 0, hh
-        end = hw, 0
-        c1 = hw, hh - 1
-        c2 = hw, int(0.25 * height)
-    xfunc, yfunc = cubic_bezier(start, end, c1, c2)
-    draw_parametrized_curve(buf, width, height, delta * supersample_factor, extra * supersample_factor, xfunc, yfunc)
+def rounded_corner(buf: SSByteArray, width: int, height: int, level: int = 1, which: str = '╭') -> None:
+    xfunc, yfunc = rectircle_equations(width, height, buf.supersample_factor, which)
+    draw_parametrized_curve(buf, width, height, level, xfunc, yfunc)
+
+
+@supersampled()
+def rounded_separator(buf: SSByteArray, width: int, height: int, level: int = 1, left: bool = True) -> None:
+    gap = thickness(level) * buf.supersample_factor
+    c1x = find_bezier_for_D(width - gap, height)
+    start = (0, 0)
+    end = (0, height - 1)
+    c1 = c1x, start[1]
+    c2 = c1x, end[1]
+    bezier_x, bezier_y = cubic_bezier(start, end, c1, c2)
+    if left:
+        draw_parametrized_curve(buf, width, height, level, bezier_x, bezier_y)
+    else:
+        mbuf = SSByteArray(width * height)
+        mbuf.supersample_factor = buf.supersample_factor
+        draw_parametrized_curve(mbuf, width, height, level, bezier_x, bezier_y)
+        for y in range(height):
+            offset = y * width
+            for src_x in range(width):
+                dest_x = width - 1 - src_x
+                buf[offset + dest_x] = mbuf[offset + src_x]
 
 
 def half_dhline(buf: BufType, width: int, height: int, level: int = 1, which: str = 'left', only: Optional[str] = None) -> Tuple[int, int]:
@@ -569,7 +622,6 @@ def shade(buf: BufType, width: int, height: int, light: bool = False, invert: bo
     dest = bytearray(width * height) if invert else buf
 
     for r in range(number_of_rows):
-        y = r * square_sz
         is_odd = r % 2 != 0
         if is_odd:
             continue
@@ -642,7 +694,7 @@ def sextant(buf: BufType, width: int, height: int, level: int = 1, which: int =
 
 @supersampled()
 def smooth_mosaic(
-    buf: BufType, width: int, height: int, level: int = 1,
+    buf: SSByteArray, width: int, height: int, level: int = 1,
     lower: bool = True, a: Tuple[float, float] = (0, 0), b: Tuple[float, float] = (0, 0)
 ) -> None:
     ax, ay = int(a[0] * (width - 1)), int(a[1] * (height - 1))
@@ -663,17 +715,32 @@ def smooth_mosaic(
                 buf[offset + x] = 255
 
 
+def eight_range(size: int, which: int) -> range:
+    thickness = max(1, size // 8)
+    block = thickness * 8
+    if block == size:
+        return range(thickness * which, thickness * (which + 1))
+    if block > size:
+        start = min(which * thickness, size - thickness)
+        return range(start, start + thickness)
+    extra = size - block
+    thicknesses = list(repeat(thickness, 8))
+    for i in (3, 4, 2, 5, 6, 1, 7, 0):  # ensures the thickness of first and last are least likely to be changed
+        if not extra:
+            break
+        extra -= 1
+        thicknesses[i] += 1
+    pos = sum(thicknesses[:which])
+    return range(pos, pos + thicknesses[which])
+
+
 def eight_bar(buf: BufType, width: int, height: int, level: int = 1, which: int = 0, horizontal: bool = False) -> None:
     if horizontal:
         x_range = range(0, width)
-        thickness = max(1, height // 8)
-        y_start = min(which * thickness, height - 2)
-        y_range = range(y_start, height if which == 7 else min(y_start + thickness, height))
+        y_range = eight_range(height, which)
     else:
         y_range = range(0, height)
-        thickness = max(1, width // 8)
-        x_start = min(which * thickness, width - 2)
-        x_range = range(x_start, width if which == 7 else min(x_start + thickness, width))
+        x_range = eight_range(width, which)
     for y in y_range:
         offset = y * width
         for x in x_range:
@@ -685,19 +752,32 @@ def eight_block(buf: BufType, width: int, height: int, level: int = 1, which: Tu
         eight_bar(buf, width, height, level, x, horizontal)
 
 
+@lru_cache(maxsize=64)
+def distribute_dots(available_space: int, num_of_dots: int) -> Tuple[Tuple[int, ...], int]:
+    dot_size = max(1, available_space // (2 * num_of_dots))
+    extra = available_space - 2 * num_of_dots * dot_size
+    gaps = list(repeat(dot_size, num_of_dots))
+    if extra > 0:
+        idx = 0
+        while extra > 0:
+            gaps[idx] += 1
+            idx = (idx + 1) % len(gaps)
+            extra -= 1
+    gaps[0] //= 2
+    summed_gaps = tuple(sum(gaps[:i + 1]) for i in range(len(gaps)))
+    return summed_gaps, dot_size
+
+
 def braille_dot(buf: BufType, width: int, height: int, col: int, row: int) -> None:
-    dot_height = max(1, height // 8)
-    dot_width = max(1, width // 4)
-    top_margin = (height - 7 * dot_height) // 2
-    left_margin = (width - 3 * dot_width) // 2
-    x_start = left_margin + (col * 2 * dot_width)
-    y_start = top_margin + (row * 2 * dot_height)
-    if y_start < height:
+    x_gaps, dot_width = distribute_dots(width, 2)
+    y_gaps, dot_height = distribute_dots(height, 4)
+    x_start = x_gaps[col] + col * dot_width
+    y_start = y_gaps[row] + row * dot_height
+    if y_start < height and x_start < width:
         for y in range(y_start, min(height, y_start + dot_height)):
-            if x_start < width:
-                offset = y * width
-                for x in range(x_start, min(width, x_start + dot_width)):
-                    buf[offset + x] = 255
+            offset = y * width
+            for x in range(x_start, min(width, x_start + dot_width)):
+                buf[offset + x] = 255
 
 
 def braille(buf: BufType, width: int, height: int, which: int = 0) -> None:
@@ -711,7 +791,7 @@ def braille(buf: BufType, width: int, height: int, which: int = 0) -> None:
             braille_dot(buf, width, height, col, row)
 
 
-box_chars: Dict[str, List[Callable]] = {
+box_chars: Dict[str, List[Callable[[BufType, int, int], Any]]] = {
     '─': [hline],
     '━': [p(hline, level=3)],
     '│': [vline],
@@ -741,15 +821,21 @@ box_chars: Dict[str, List[Callable]] = {
     '╾': [p(half_hline, level=3), p(half_hline, which='right')],
     '╿': [p(half_vline, level=3), p(half_vline, which='bottom')],
     '': [triangle],
-    '': [p(triangle, left=False)],
-    '': [D],
-    '': [p(D, left=False)],
     '': [p(half_cross_line, which='tl'), p(half_cross_line, which='bl')],
+    '': [p(triangle, left=False)],
     '': [p(half_cross_line, which='tr'), p(half_cross_line, which='br')],
+    '': [D],
+    '': [rounded_separator],
+    '': [p(D, left=False)],
+    '': [p(rounded_separator, left=False)],
     '': [p(corner_triangle, corner='bottom-left')],
+    '': [cross_line],
     '': [p(corner_triangle, corner='bottom-right')],
+    '': [p(cross_line, left=False)],
     '': [p(corner_triangle, corner='top-left')],
+    '': [p(cross_line, left=False)],
     '': [p(corner_triangle, corner='top-right')],
+    '': [cross_line],
     '═': [dhline],
     '║': [dvline],
 
@@ -813,8 +899,8 @@ box_chars: Dict[str, List[Callable]] = {
 
     '🬼': [p(smooth_mosaic, a=(0, 0.75), b=(0.5, 1))],
     '🬽': [p(smooth_mosaic, a=(0, 0.75), b=(1, 1))],
-    '🬾': [p(smooth_mosaic, a=(0, 0.5), b=(0.5, 1))],
-    '🬿': [p(smooth_mosaic, a=(0, 0.5), b=(1, 1))],
+    '🬾': [p(smooth_mosaic, a=(0, 0.25), b=(0.5, 1))],
+    '🬿': [p(smooth_mosaic, a=(0, 0.25), b=(1, 1))],
     '🭀': [p(smooth_mosaic, a=(0, 0), b=(0.5, 1))],
 
     '🭁': [p(smooth_mosaic, a=(0, 0.25), b=(0.5, 0))],
@@ -932,7 +1018,7 @@ for starts, func, pattern in (
 
 for chars, func_ in (('╒╕╘╛', dvcorner), ('╓╖╙╜', dhcorner), ('╔╗╚╝', dcorner), ('╟╢╤╧', dpip)):
     for ch in chars:
-        box_chars[ch] = [p(cast(Callable, func_), which=ch)]
+        box_chars[ch] = [p(func_, which=ch)]
 
 for i in range(256):
     box_chars[chr(0x2800 + i)] = [p(braille, which=i)]
@@ -940,10 +1026,9 @@ for i in range(256):
 
 c = 0x1fb00
 for i in range(1, 63):
-    if i in (20, 40):
-        continue
-    box_chars[chr(c)] = [p(sextant, which=i)]
-    c += 1
+    if i not in (21, 42):
+        box_chars[chr(c)] = [p(sextant, which=i)]
+        c += 1
 
 for i in range(1, 7):
     box_chars[chr(0x1fb6f + i)] = [p(eight_bar, which=i)]
@@ -969,8 +1054,9 @@ def render_missing_glyph(buf: BufType, width: int, height: int) -> None:
 
 def test_char(ch: str, sz: int = 48) -> None:
     # kitty +runpy "from kitty.fonts.box_drawing import test_char; test_char('XXX')"
-    from .render import display_bitmap, setup_for_testing
     from kitty.fast_data_types import concat_cells, set_send_sprite_to_gpu
+
+    from .render import display_bitmap, setup_for_testing
     with setup_for_testing('monospace', sz) as (_, width, height):
         buf = bytearray(width * height)
         try:
@@ -987,10 +1073,11 @@ def test_char(ch: str, sz: int = 48) -> None:
             set_send_sprite_to_gpu(None)
 
 
-def test_drawing(sz: int = 48, family: str = 'monospace') -> None:
-    from .render import display_bitmap, setup_for_testing
+def test_drawing(sz: int = 48, family: str = 'monospace', start: int = 0x2500, num_rows: int = 10, num_cols: int = 16) -> None:
     from kitty.fast_data_types import concat_cells, set_send_sprite_to_gpu
 
+    from .render import display_bitmap, setup_for_testing
+
     with setup_for_testing(family, sz) as (_, width, height):
         space = bytearray(width * height)
 
@@ -1005,14 +1092,14 @@ def test_drawing(sz: int = 48, family: str = 'monospace') -> None:
                 return cell
             return space
 
-        pos = 0x2500
+        pos = start
         rows = []
         space_row = join_cells(repeat(space, 32))
 
         try:
-            for r in range(10):
+            for r in range(num_rows):
                 row = []
-                for i in range(16):
+                for i in range(num_cols):
                     row.append(render_chr(chr(pos)))
                     row.append(space)
                     pos += 1
@@ -1021,7 +1108,7 @@ def test_drawing(sz: int = 48, family: str = 'monospace') -> None:
             rgb_data = b''.join(rows)
             width *= 32
             height *= len(rows)
-            assert len(rgb_data) == width * height * 4, '{} != {}'.format(len(rgb_data), width * height * 4)
+            assert len(rgb_data) == width * height * 4, f'{len(rgb_data)} != {width * height * 4}'
             display_bitmap(rgb_data, width, height)
         finally:
             set_send_sprite_to_gpu(None)
