@@ -5,28 +5,29 @@
 import re
 import sys
 from functools import lru_cache
-from typing import (
-    Any, Callable, Container, Dict, FrozenSet, Iterable, Iterator, List,
-    NamedTuple, Optional, Sequence, Tuple, Union
-)
+from typing import Any, Callable, Container, Dict, FrozenSet, Iterable, Iterator, List, NamedTuple, Optional, Sequence, Tuple, Union
 
 import kitty.fast_data_types as defines
 from kitty.conf.utils import (
-    CurrentlyParsing, KeyAction, KeyFuncWrapper, currently_parsing,
-    positive_float, positive_int, python_string, to_bool, to_cmdline, to_color,
-    uniq, unit_float
+    CurrentlyParsing,
+    KeyAction,
+    KeyFuncWrapper,
+    currently_parsing,
+    positive_float,
+    positive_int,
+    python_string,
+    to_bool,
+    to_cmdline,
+    to_color,
+    uniq,
+    unit_float,
 )
 from kitty.constants import is_macos
-from kitty.fast_data_types import (
-    CURSOR_BEAM, CURSOR_BLOCK, CURSOR_UNDERLINE, Color
-)
-from kitty.fonts import FontFeature
-from kitty.key_names import (
-    character_key_name_aliases, functional_key_name_aliases,
-    get_key_name_lookup
-)
+from kitty.fast_data_types import CURSOR_BEAM, CURSOR_BLOCK, CURSOR_UNDERLINE, Color, SingleKey
+from kitty.fonts import FontFeature, FontModification, ModificationType, ModificationUnit, ModificationValue
+from kitty.key_names import character_key_name_aliases, functional_key_name_aliases, get_key_name_lookup
 from kitty.rgb import color_as_int
-from kitty.types import FloatEdges, MouseEvent, SingleKey
+from kitty.types import FloatEdges, MouseEvent
 from kitty.utils import expandvars, log_error, resolve_abs_or_config_path
 
 KeyMap = Dict[SingleKey, str]
@@ -64,7 +65,7 @@ def shlex_parse(func: str, rest: str) -> FuncArgsType:
 
 
 def parse_send_text_bytes(text: str) -> bytes:
-    return python_string(text).encode('utf-8')
+    return defines.expand_ansi_c_escapes(text).encode('utf-8')
 
 
 @func_with_args('send_text')
@@ -91,6 +92,20 @@ def kitten_parse(func: str, rest: str) -> FuncArgsType:
     return func, [args[0]] + (to_cmdline(args[1]) if len(args) > 1 else [])
 
 
+@func_with_args('open_url')
+def open_url_parse(func: str, rest: str) -> FuncArgsType:
+    from urllib.parse import urlparse
+    url = ''
+    try:
+        url = python_string(rest)
+        tokens = urlparse(url)
+        if not all((tokens.scheme, tokens.netloc,)):
+            raise ValueError('Invalid URL')
+    except Exception:
+        log_error('Ignoring invalid URL string: ' + rest)
+    return func, (url,)
+
+
 @func_with_args('goto_tab')
 def goto_tab_parse(func: str, rest: str) -> FuncArgsType:
     args = (max(0, int(rest)), )
@@ -119,7 +134,7 @@ def detach_tab_parse(func: str, rest: str) -> FuncArgsType:
     return func, (rest,)
 
 
-@func_with_args('set_background_opacity', 'goto_layout', 'toggle_layout', 'kitty_shell')
+@func_with_args('set_background_opacity', 'goto_layout', 'toggle_layout', 'kitty_shell', 'show_kitty_doc', 'set_tab_title')
 def simple_parse(func: str, rest: str) -> FuncArgsType:
     return func, [rest]
 
@@ -187,7 +202,7 @@ def paste_from_buffer(func: str, rest: str) -> FuncArgsType:
 def paste_parse(func: str, rest: str) -> FuncArgsType:
     text = ''
     try:
-        text = python_string(rest)
+        text = defines.expand_ansi_c_escapes(rest)
     except Exception:
         log_error('Ignoring invalid paste string: ' + rest)
     return func, [text]
@@ -265,7 +280,7 @@ def remote_control(func: str, rest: str) -> FuncArgsType:
     return func, args
 
 
-@func_with_args('nth_os_window', 'nth_window', 'scroll_to_prompt', 'visual_window_select_action_trigger')
+@func_with_args('nth_os_window', 'nth_window', 'scroll_to_prompt', 'visual_window_select_action_trigger', 'next_layout')
 def single_integer_arg(func: str, rest: str) -> FuncArgsType:
     try:
         num = int(rest)
@@ -276,6 +291,16 @@ def single_integer_arg(func: str, rest: str) -> FuncArgsType:
     return func, [num]
 
 
+@func_with_args('sleep')
+def sleep(func: str, sleep_time: str) -> FuncArgsType:
+    mult = 1
+    sleep_time = sleep_time or '1'
+    if sleep_time[-1] in 'shmd':
+        mult = {'s': 1, 'm': 60, 'h': 3600, 'd': 24 * 3600}[sleep_time[-1]]
+        sleep_time = sleep_time[:-1]
+    return func, [abs(float(sleep_time)) * mult]
+
+
 @func_with_args('disable_ligatures_in')
 def disable_ligatures_in(func: str, rest: str) -> FuncArgsType:
     parts = rest.split(maxsplit=1)
@@ -304,13 +329,13 @@ def parse_marker_spec(ftype: str, parts: Sequence[str]) -> Tuple[str, Union[str,
         if ftype.startswith('i'):
             flags |= re.IGNORECASE
         if not parts or len(parts) % 2 != 0:
-            raise ValueError('No color specified in marker: {}'.format(' '.join(parts)))
+            raise ValueError('Mark group number and text/regex are not specified in pairs: {}'.format(' '.join(parts)))
         ans = []
         for i in range(0, len(parts), 2):
             try:
                 color = max(1, min(int(parts[i]), 3))
             except Exception:
-                raise ValueError(f'color {parts[i]} in marker specification is not an integer')
+                raise ValueError(f'Mark group in marker specification is not an integer: {parts[i]}')
             sspec = parts[i + 1]
             if 'regex' not in ftype:
                 sspec = re.escape(sspec)
@@ -432,26 +457,6 @@ def parse_shortcut(sc: str) -> SingleKey:
     return SingleKey(mods, is_native, key or 0)
 
 
-def adjust_line_height(x: str) -> Union[int, float]:
-    if x.endswith('%'):
-        ans = float(x[:-1].strip()) / 100.0
-        if ans < 0:
-            log_error('Percentage adjustments of cell sizes must be positive numbers')
-            return 0
-        return ans
-    return int(x)
-
-
-def adjust_baseline(x: str) -> Union[int, float]:
-    if x.endswith('%'):
-        ans = float(x[:-1].strip()) / 100.0
-        if abs(ans) > 1:
-            log_error('Percentage adjustments of the baseline cannot exceed 100%')
-            return 0
-        return ans
-    return int(x)
-
-
 def to_font_size(x: str) -> float:
     return max(MINIMUM_FONT_SIZE, float(x))
 
@@ -678,10 +683,18 @@ def config_or_absolute_path(x: str, env: Optional[Dict[str, str]] = None) -> Opt
     return resolve_abs_or_config_path(x, env)
 
 
-def allow_remote_control(x: str) -> str:
-    if x != 'socket-only':
-        x = 'y' if to_bool(x) else 'n'
-    return x
+def remote_control_password(val: str, current_val: Dict[str, str]) -> Iterable[Tuple[str, Sequence[str]]]:
+    val = val.strip()
+    if val:
+        parts = to_cmdline(val, expand=False)
+        if parts[0].startswith('-'):
+            # this is done so in the future we can add --options to the cmd
+            # line of remote_control_password
+            raise ValueError('Passwords are not allowed to start with hyphens, ignoring this password')
+        if len(parts) == 1:
+            yield parts[0], tuple()
+        else:
+            yield parts[0], tuple(parts[1:])
 
 
 def clipboard_control(x: str) -> Tuple[str, ...]:
@@ -746,6 +759,10 @@ def tab_bar_margin_height(x: str) -> TabBarMarginHeight:
     return TabBarMarginHeight(next(ans), next(ans))
 
 
+def clone_source_strategies(x: str) -> FrozenSet[str]:
+    return frozenset({'venv', 'conda', 'path', 'env_var'} & set(x.lower().split(',')))
+
+
 def clear_all_mouse_actions(val: str, dict_with_parse_results: Optional[Dict[str, Any]] = None) -> bool:
     ans = to_bool(val)
     if ans and dict_with_parse_results is not None:
@@ -779,6 +796,44 @@ def font_features(val: str) -> Iterable[Tuple[str, Tuple[FontFeature, ...]]]:
         yield parts[0], tuple(features)
 
 
+def modify_font(val: str) -> Iterable[Tuple[str, FontModification]]:
+    parts = val.split()
+    pos, plen = 0, len(parts)
+    if plen < 2:
+        log_error(f"Ignoring invalid modify_font: {val}")
+        return
+    mtype: Optional[ModificationType] = getattr(ModificationType, parts[pos], None)
+    if mtype is None:
+        log_error(f"Ignoring invalid modify_font with unknown modification type: {parts[pos]}")
+        return
+    pos += 1
+    font_name = ''
+    if mtype is ModificationType.size:
+        font_name = parts[pos]
+        pos += 1
+    if plen - pos < 1:
+        log_error(f"Ignoring invalid modify_font: {val}")
+        return
+    sz = parts[pos]
+    pos += 1
+    munit = ModificationUnit.pt
+    if sz.endswith('%'):
+        munit = ModificationUnit.percent
+        sz = sz[:-1]
+    elif sz.endswith('px'):
+        munit = ModificationUnit.pixel
+        sz = sz[:-2]
+    try:
+        mvalue = float(sz)
+    except Exception:
+        log_error(f'Ignoring modify_font with invalid size: {sz}')
+        return
+    key = mtype.name
+    if font_name:
+        key += f':{font_name}'
+    yield key, FontModification(mtype, ModificationValue(mvalue, munit), font_name)
+
+
 def env(val: str, current_val: Dict[str, str]) -> Iterable[Tuple[str, str]]:
     val = val.strip()
     if val:
@@ -799,12 +854,14 @@ def store_multiple(val: str, current_val: Container[str]) -> Iterable[Tuple[str,
         yield val, val
 
 
+allowed_shell_integration_values = frozenset({'enabled', 'disabled', 'no-rc', 'no-cursor', 'no-title', 'no-prompt-mark', 'no-complete', 'no-cwd'})
+
+
 def shell_integration(x: str) -> FrozenSet[str]:
-    s = frozenset({'enabled', 'disabled', 'no-rc', 'no-cursor', 'no-title', 'no-prompt-mark', 'no-complete', 'no-cwd'})
     q = frozenset(x.lower().split())
-    if not q.issubset(s):
-        log_error(f'Invalid shell integration options: {q - s}, ignoring')
-        return q & s or frozenset({'invalid'})
+    if not q.issubset(allowed_shell_integration_values):
+        log_error(f'Invalid shell integration options: {q - allowed_shell_integration_values}, ignoring')
+        return q & allowed_shell_integration_values or frozenset({'invalid'})
     return q
 
 
@@ -973,6 +1030,10 @@ class BaseDefinition:
         return f'{self.__class__.__name__}({", ".join(kwds)})'
 
 
+def resolve_key_mods(kitty_mod: int, mods: int) -> int:
+    return SingleKey(mods=mods).resolve_kitty_mod(kitty_mod).mods
+
+
 class MouseMapping(BaseDefinition):
     map_type: str = 'mouse_map'
 
@@ -991,7 +1052,7 @@ class MouseMapping(BaseDefinition):
 
     def resolve_and_copy(self, kitty_mod: int) -> 'MouseMapping':
         ans = MouseMapping(
-            self.button, defines.resolve_key_mods(kitty_mod, self.mods), self.repeat_count, self.grabbed,
+            self.button, resolve_key_mods(kitty_mod, self.mods), self.repeat_count, self.grabbed,
             self.definition)
         ans.definition_location = self.definition_location
         return ans
@@ -1017,8 +1078,7 @@ class KeyDefinition(BaseDefinition):
 
     def resolve_and_copy(self, kitty_mod: int) -> 'KeyDefinition':
         def r(k: SingleKey) -> SingleKey:
-            mods = defines.resolve_key_mods(kitty_mod, k.mods)
-            return k._replace(mods=mods)
+            return k.resolve_kitty_mod(kitty_mod)
         ans = KeyDefinition(
             self.is_sequence, r(self.trigger), tuple(map(r, self.rest)),
             self.definition
@@ -1144,3 +1204,16 @@ def deprecated_send_text(key: str, val: str, ans: Dict[str, Any]) -> None:
     key_str = f'{sc} send_text {mode} {text}'
     for k in parse_map(key_str):
         ans['map'].append(k)
+
+
+def deprecated_adjust_line_height(key: str, x: str, opts_dict: Dict[str, Any]) -> None:
+    fm = {'adjust_line_height': 'cell_height', 'adjust_baseline': 'baseline', 'adjust_column_width': 'cell_width'}[key]
+    mtype = getattr(ModificationType, fm)
+    if x.endswith('%'):
+        ans = float(x[:-1].strip())
+        if ans < 0:
+            log_error(f'Percentage adjustments of {key} must be positive numbers')
+            return
+        opts_dict['modify_font'][fm] = FontModification(mtype, ModificationValue(ans, ModificationUnit.percent))
+    else:
+        opts_dict['modify_font'][fm] = FontModification(mtype, ModificationValue(int(x), ModificationUnit.pixel))
